/*
 * support/export/xtab.c
 *
 * Interface to the xtab file.
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include <stdlib.h>
#include "xmalloc.h"
#include "nfslib.h"
#include "exportfs.h"
#include "xio.h"
#include "xlog.h"

int
xtab_read(void)
{
	struct exportent	*xp;
	nfs_export		*exp;
	int			lockid;

	if ((lockid = xflock(_PATH_XTAB, "r")) < 0)
		return 0;
	setexportent(_PATH_XTAB, "r");
	while ((xp = getexportent()) != NULL) {
		if (!(exp = export_lookup(xp->e_hostname, xp->e_path)) &&
		    !(exp = export_create(xp))) {
			continue;
		}
		exp->m_xtabent = 1;
		exp->m_mayexport = 1;
	}
	endexportent();
	xfunlock(lockid);

	return 0;
}

int
xtab_write(int append)
{
	struct exportent	xe;
	nfs_export		*exp;
	int			lockid, i;

	if ((lockid = xflock(_PATH_XTAB, "w")) < 0) {
		xlog(L_ERROR, "can't lock %s for writing", _PATH_XTAB);
		return 0;
	}
	if (append)
		setexportent(_PATH_XTAB, "a");
	else
		setexportent(_PATH_XTABTMP, "w");

	for (i = 0; i < MCL_MAXTYPES; i++) {
		for (exp = exportlist[i]; exp; exp = exp->m_next) {
			if ((append && exp->m_xtabent) ||
			    (!exp->m_xtabent && !exp->m_addxtab))
				continue;

			/* write out the export entry using the FQDN */
			xe = exp->m_export;
			strcpy(xe.e_hostname, exp->m_client->m_hostname);
			putexportent(&xe);
		}
	}
	endexportent();

	if (!append)
		rename(_PATH_XTABTMP, _PATH_XTAB);

	xfunlock(lockid);

	return 1;
}

void
xtab_append(nfs_export *exp)
{
	struct exportent xe;
	int		lockid;

	if ((lockid = xflock(_PATH_XTAB, "w")) < 0)
		return;
	setexportent(_PATH_XTAB, "a");
	xe = exp->m_export;
	strcpy(xe.e_hostname, exp->m_client->m_hostname);
	putexportent(&xe);
	endexportent();
	xfunlock(lockid);
	exp->m_xtabent = 1;
}

