/*
 * support/include/exportfs.h
 *
 * Declarations for exportfs and mountd
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef EXPORTFS_H
#define EXPORTFS_H

#include <linux/nfsd/syscall.h>
#include <netdb.h>
#include "nfslib.h"

enum {
	MCL_FQDN = 0, MCL_SUBNETWORK, MCL_WILDCARD,
	MCL_NETGROUP, MCL_ANONYMOUS,
	MCL_MAXTYPES
};

typedef struct mclient {
	struct mclient *	m_next;
	char			m_hostname[NFSCLNT_IDMAX+1];
	int			m_type;
	int			m_naddr;
	struct in_addr		m_addrlist[NFSCLNT_ADDRMAX];
	struct in_addr		m_mask;		/* for addr/mask entries */
	int			m_exported;	/* exported to nfsd */
	int			m_count;
} nfs_client;

typedef struct mexport {
	struct mexport *	m_next;
	struct mclient *	m_client;
	struct exportent	m_export;
	int			m_exported : 1,	/* known to knfsd */
				m_xtabent  : 1,	/* xtab entry exists */
				m_mayexport: 1,	/* derived from xtabbed */
				m_addxtab  : 1;	/* to be added to xtab */
} nfs_export;

extern nfs_client *		clientlist[MCL_MAXTYPES];
extern nfs_export *		exportlist[MCL_MAXTYPES];

nfs_client *			client_lookup(char *hname);
nfs_client *			client_find(struct hostent *);
void				client_add(nfs_client *);
nfs_client *			client_dup(nfs_client *, struct hostent *);
int				client_gettype(char *hname);
int				client_check(nfs_client *, struct hostent *);
int				client_match(nfs_client *, char *hname);
void				client_release(nfs_client *);
void				client_freeall(void);

int				export_read(char *fname);
void				export_add(nfs_export *);
nfs_export *			export_lookup(char *hname, char *path);
nfs_export *			export_find(struct hostent *, char *path);
nfs_export *			export_create(struct exportent *);
void				export_freeall(void);
int				export_export(nfs_export *);
int				export_unexport(nfs_export *);

int				xtab_read(void);
int				xtab_write(int append);
void				xtab_append(nfs_export *);

struct nfskey *			key_lookup(char *hname);

#endif /* EXPORTFS_H */

