.\"
.\" exportfs(8)
.\" 
.\" Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
.TH exportfs 8 "15 Feb 1996"
.SH NAME
exportfs \- maintain list of NFS exported file systems
.SH SYNOPSIS
.BI "/usr/sbin/exportfs [-av] [-o " "options,.." "] [" "client:/path" " ..]
.br
.BI "/usr/sbin/exportfs [-av] -u [" "client:/path" " ..]
.br
.BI "/usr/sbin/exportfs [-v]
.br
.SH DESCRIPTION
The
.B exportfs
command is used to maintain the current table of exported file systems for
NFS. This list is kept in a separate file named
.BR /var/lib/nfs/xtab .
Normally, this file is initialized with the list of all file systems named in
.B /etc/exports 
by invoking
.BR "exportfs -a" ,
and modified by
.B mountd(8)
whenever receiving a mount request from a host that is matched by a network
or domain export.
.P
However, administrators can choose to add and delete individual file systems
without modifying
.B /etc/exports
using
.BR exportfs .
.P
.SH OPTIONS
.TP 
.B -a
Export or unexport all directories.
.TP
.BI "-o " options,...
Specify a list of export options in the same manner as in
.BR exports(5) .
.TP
.B -u
Unexport one or more directories.
.TP
.B -v
Be verbose. When exporting or unexporting, show what's going on. When
displaying the current export list, also display the list of export
options.
.SH DISCUSSION
.\" -------------------- Exporting Directories --------------------
.SS Exporting Directories
The first synopsis shows how to invoke the command when adding new
entries to the export table.  When using 
.BR "exportfs -a" ,
all directories in
.B exports(5)
are added to
.B xtab
and the resulting list is pushed into the kernel.
.P
The
.I host:/path
argument specifies the directory to export along with the host or hosts to
export it to. All formats described in
.B exports(5)
are supported; to export a directory to the world, simply specify
.IR /path .
.P
The export options for a particular host/directory pair derive from
several sources.  The set of default options can be overridden by
entries in
.BR /usr/etc/exports " and " /var/lib/nfs/xtab ,
if they exists.
In addition, the administrator may overide any options from these sources
using the
.B -o
argument which takes a comma-separated list of options in the same fashion
as one would specify them in
.BR exports(5) .
Thus,
.B exportfs
can also be used to modify the export options of an already exported
directory.
.P
Modifications of the kernel export table used by
.B nfsd(8)
take place immediately after parsing the command line and updating the
.B xtab
file.
.\" -------------------- Unexporting Directories ------------------
.SS Unexporting Directories
The second synopsis shows how to unexported a currently exported directory.
When using
.BR "exportfs -ua" ,
all entries listed in
.B xtab
are removed from the kernel export tables, and the file is cleared. This
effectively shuts down all NFS activity.
.P
To remove individial export entries, one can specify a
.I host:/path
pair. This deletes the specified entry from
.B xtab
and removes the corresponding kernel entry.
.P
.\" -------------------- Dumping the Export Table -----------------
.SS Dumping the Export Table 
Invoking
.B exportfs
without further options shows the current list of exported file systems.
When giving the
.B -v
option, the list of flags pertaining to each export are shown in addition.
.\" -------------------- EXAMPLES ---------------------------------
.SH EXAMPLES
The following adds all directories listed in
.B /etc/exports to /var/lib/nfs/xtab
and pushes the resulting export entries into the kernel:
.P
.nf
.B "# exportfs -a
.fi
.P
To export the
.B /usr/tmp
directory to host 
.BR djando ,
allowing asynchronous writes, one would do this:
.P
.nf
.B "# exportfs django:/usr/tmp -o async
.fi
.P
If the directory was already exported, 
.\" -------------------- DEPENDENCIES -----------------------------
.SH DEPENDENCIES
Exporting to IP networks, DNS and NIS domains does not enable clients
from these groups to access NFS immediately; rather, these sorts of
exports are hints to
.B mountd(8)
to grant any mount requests from these clients.
This is usually not a big problem, because any existing mounts are preserved
in
.B xtab
across reboots.
.P
When unexporting a network or domain entry, any current exports to members
of this group are unaffected. The reason for this is that 
.B exportfs
cannot tell whether these specific exports were created as a result of the
group export, or whether they have been exported independently.
.P
.\" -------------------- SEE ALSO --------------------------------
.SH SEE ALSO
.BR exports(5) ", " mountd(8)
.\" -------------------- AUTHOR ----------------------------------
.SH AUTHOR
Olaf Kirch, <okir@monad.swb.de>
